<?php
/**
 * @file
 * Defines the CommerceBookingTicket class.
 */

/**
 * The class used for commerce_booking_ticket entities.
 */
class CommerceBookingTicket extends Entity {

  /**
   * The ticket id.
   *
   * @var integer
   */
  public $ticket_id;

  /**
   * The name of the ticket type.
   *
   * @var string
   */
  public $type;

  /**
   * The status of the ticket.
   *
   * @var string
   *
   * @see commerce_booking_ticket_status_info.
   */
  public $status;

  /**
   * Overrides Entity::construct().
   */
  public function __construct($values = array()) {
    // Set a default value on the status column if none is set.
    if (empty($this->status) && empty($values['status'])) {
      $status = 'pending';
      foreach (commerce_booking_ticket_status_info() as $settings) {
        if (!empty($settings['default'])) {
          $status = $settings['name'];
        }
      }

      $values['status'] = $status;
    }

    parent::__construct($values, 'commerce_booking_ticket');
  }

  /**
   * Overides Entity::defaultUri().
   */
  public function defaultUri() {
    return array(
      'path' => 'ticket/' . $this->ticket_id,
      'options' => array(''),
    );
  }

  /**
   * Get a human readable label for the entity.
   *
   * @todo: implement well.
   */
  public function defaultLabel() {
    if (!empty($this->label)) {
      return $this->label;
    }

    $entity_type = commerce_booking_get_event_entity_type($this);
    $entity = commerce_booking_get_event_entity($this);
    $label = entity_label($entity_type, $entity);
    return t('Ticket for @event', array('@event' => $label));
  }
}
